@@ -180,6 +180,9 @@ connection_cell_ids = MethodSpec(name='connection_cell_ids',
         
         return self._get_cell_id(self.pre_cell_id)
         
+    def get_post_cell_id(self):
+        
+        return self._get_cell_id(self.post_cell_id)
 
     def get_pre_segment_id(self):
         
@@ -198,10 +201,6 @@ connection_cell_ids = MethodSpec(name='connection_cell_ids',
         return float(self.post_fraction_along)
         
         
-    def get_post_cell_id(self):
-        
-        return self._get_cell_id(self.post_cell_id)
-        
     def get_pre_info(self):
         
         return str(self.get_pre_cell_id())+(':'+str(self.get_pre_segment_id())+'('+ 'PERCENTAGE.5f'PERCENTAGEself.get_pre_fraction_along()+')' if self.get_pre_segment_id()!=0 or self.get_pre_fraction_along()!=0.5 else '')
@@ -225,7 +224,7 @@ connection_wd_cell_ids = MethodSpec(name='connection_wd_cell_ids',
         
     def __str__(self):
         
-        return "Connection "+str(self.id)+": "+str(self.get_pre_cell_id())+" -> "+str(self.get_post_cell_id())+ \
+        return "Connection "+str(self.id)+": "+str(self.get_pre_info())+" -> "+str(self.get_post_info())+ \
             ", weight: "+'PERCENTAGEf' PERCENTAGE (float(self.weight))+", delay: "+'PERCENTAGE.5f' PERCENTAGE (self.get_delay_in_ms())+" ms"
             
     def get_delay_in_ms(self):
@@ -248,18 +247,10 @@ elec_connection_instance_cell_ids = MethodSpec(name='elec_connection_instance_ce
             return int(id_string.split('[')[1].split(']')[0])
         else:
             return int(id_string.split('/')[2])
-            
-    def get_pre_cell_id(self):
-        
-        return self._get_cell_id(self.pre_cell)
-        
-    def get_post_cell_id(self):
-        
-        return self._get_cell_id(self.post_cell)
         
     def __str__(self):
         
-        return "Electrical Connection (Instance based) "+str(self.id)+": "+str(self.get_pre_cell_id())+" -> "+str(self.get_post_cell_id())+ \
+        return "Electrical Connection (Instance based) "+str(self.id)+": "+str(self.get_pre_info())+" -> "+str(self.get_post_info())+ \
             ", synapse: "+str(self.synapse)
             
         
@@ -269,6 +260,24 @@ elec_connection_instance_cell_ids = MethodSpec(name='elec_connection_instance_ce
   
 METHOD_SPECS+=(elec_connection_instance_cell_ids,)
 
+elec_connection_instance_w = MethodSpec(name='elec_connection_instance_w',
+    source='''\
+        
+    def get_weight(self):
+        
+        return float(self.weight) if self.weight else 1.0
+        
+    def __str__(self):
+        
+        return "Electrical Connection (Instance based & weight) "+str(self.id)+": "+str(self.get_pre_info())+" -> "+str(self.get_post_info())+ \
+            ", synapse: "+str(self.synapse) + ", weight: "+'PERCENTAGE.6f'PERCENTAGEself.get_weight()
+            
+    ''',
+    class_names=("ElectricalConnectionInstanceW")
+    )
+  
+METHOD_SPECS+=(elec_connection_instance_w,)
+
 elec_connection_cell_ids = MethodSpec(name='elec_connection_cell_ids',
     source='''\
         
@@ -283,9 +292,35 @@ elec_connection_cell_ids = MethodSpec(name='elec_connection_cell_ids',
         
         return self._get_cell_id(self.post_cell)
         
+    def get_pre_segment_id(self):
+        
+        return int(self.pre_segment)
+        
+    def get_post_segment_id(self):
+        
+        return int(self.post_segment)
+
+    def get_pre_fraction_along(self):
+        
+        return float(self.pre_fraction_along)
+        
+    def get_post_fraction_along(self):
+        
+        return float(self.post_fraction_along)
+        
+        
+    def get_pre_info(self):
+        
+        return str(self.get_pre_cell_id())+(':'+str(self.get_pre_segment_id())+'('+ 'PERCENTAGE.5f'PERCENTAGEself.get_pre_fraction_along()+')' if self.get_pre_segment_id()!=0 or self.get_pre_fraction_along()!=0.5 else '')
+        
+    def get_post_info(self):
+        
+        return str(self.get_post_cell_id())+(':'+str(self.get_post_segment_id())+'('+ 'PERCENTAGE.5f'PERCENTAGEself.get_post_fraction_along()+')' if self.get_post_segment_id()!=0 or self.get_post_fraction_along()!=0.5 else '')
+        
+        
     def __str__(self):
         
-        return "Electrical Connection "+str(self.id)+": "+str(self.get_pre_cell_id())+" -> "+str(self.get_post_cell_id())+ \
+        return "Electrical Connection "+str(self.id)+": "+str(self.get_pre_info())+" -> "+str(self.get_post_info())+ \
             ", synapse: "+str(self.synapse)
             
         
@@ -295,14 +330,132 @@ elec_connection_cell_ids = MethodSpec(name='elec_connection_cell_ids',
   
 METHOD_SPECS+=(elec_connection_cell_ids,)
 
+cont_connection_instance_cell_ids = MethodSpec(name='cont_connection_instance_cell_ids',
+    source='''\
+        
+    def _get_cell_id(self, id_string):
+        if '[' in id_string:
+            return int(id_string.split('[')[1].split(']')[0])
+        else:
+            return int(id_string.split('/')[2])
+           
+        
+    def __str__(self):
+        
+        return "Continuous Connection (Instance based) "+str(self.id)+": "+str(self.get_pre_info())+" -> "+str(self.get_post_info())+ \
+            ", pre comp: "+str(self.pre_component)+", post comp: "+str(self.post_component)
+            
+        
+    ''',
+    class_names=("ContinuousConnectionInstance")
+    )
+  
+METHOD_SPECS+=(cont_connection_instance_cell_ids,)
 
-location = MethodSpec(name='location',
+cont_connection_instance_w = MethodSpec(name='cont_connection_instance_w',
+    source='''\
+        
+    def get_weight(self):
+        
+        return float(self.weight) if self.weight else 1.0
+        
+    def __str__(self):
+        
+        return "Continuous Connection (Instance based & weight) "+str(self.id)+": "+str(self.get_pre_info())+" -> "+str(self.get_post_info())+ \
+            ", pre comp: "+str(self.pre_component)+", post comp: "+str(self.post_component)+", weight: "+'PERCENTAGE.6f'PERCENTAGEself.get_weight()
+            
+        
+    ''',
+    class_names=("ContinuousConnectionInstanceW")
+    )
+  
+METHOD_SPECS+=(cont_connection_instance_w,)
+
+cont_connection_cell_ids = MethodSpec(name='cont_connection_cell_ids',
+    source='''\
+        
+    def _get_cell_id(self, id_string):
+            return int(float(id_string))
+            
+ 
+    def get_pre_cell_id(self):
+        
+        return self._get_cell_id(self.pre_cell)
+        
+    def get_post_cell_id(self):
+        
+        return self._get_cell_id(self.post_cell)
+        
+    def get_pre_segment_id(self):
+        
+        return int(self.pre_segment)
+        
+    def get_post_segment_id(self):
+        
+        return int(self.post_segment)
+
+    def get_pre_fraction_along(self):
+        
+        return float(self.pre_fraction_along)
+        
+    def get_post_fraction_along(self):
+        
+        return float(self.post_fraction_along)
+        
+        
+    def get_pre_info(self):
+        
+        return str(self.get_pre_cell_id())+(':'+str(self.get_pre_segment_id())+'('+ 'PERCENTAGE.5f'PERCENTAGEself.get_pre_fraction_along()+')' if self.get_pre_segment_id()!=0 or self.get_pre_fraction_along()!=0.5 else '')
+        
+    def get_post_info(self):
+        
+        return str(self.get_post_cell_id())+(':'+str(self.get_post_segment_id())+'('+ 'PERCENTAGE.5f'PERCENTAGEself.get_post_fraction_along()+')' if self.get_post_segment_id()!=0 or self.get_post_fraction_along()!=0.5 else '')
+        
+        
+    def __str__(self):
+        
+        return "Continuous Connection "+str(self.id)+": "+str(self.get_pre_info())+" -> "+str(self.get_post_info())+ \
+            ", pre comp: "+str(self.pre_component)+", post comp: "+str(self.post_component)
+            
+        
+    ''',
+    class_names=("ContinuousConnection")
+    )
+  
+METHOD_SPECS+=(cont_connection_cell_ids,)
+
+
+instance = MethodSpec(name='instance',
     source='''\
 
         
     def __str__(self):
         
-        return "("+ 'PERCENTAGE.4f' PERCENTAGE self.x+","+ 'PERCENTAGE.4f' PERCENTAGE self.y+","+ 'PERCENTAGE.4f' PERCENTAGE self.z+")"
+        return "Instance "+str(self.id)+ (" at location: "+str(self.location) if self.location else "")
+        
+    def __repr__(self):
+    
+        return str(self)
+    
+''',
+    class_names=("Instance")
+    )
+METHOD_SPECS+=(instance,)
+
+
+location = MethodSpec(name='location',
+    source='''\
+
+    def _format(self,value):
+    
+        if int(value)==value:
+            return str(int(value))
+        else:
+            return 'PERCENTAGE.4f' PERCENTAGE value
+        
+    def __str__(self):
+        
+        return "("+ self._format(self.x) +", "+ self._format(self.y) +", "+ self._format(self.z) +")"
         
     def __repr__(self):
     
@@ -313,6 +466,9 @@ location = MethodSpec(name='location',
     )
 METHOD_SPECS+=(location,)
 
+
+
+
 input_cell_ids = MethodSpec(name='input_cell_ids',
     source='''\
 
@@ -345,6 +501,24 @@ input_cell_ids = MethodSpec(name='input_cell_ids',
 METHOD_SPECS+=(input_cell_ids,)
 
 
+input_w = MethodSpec(name='input_w',
+    source='''\
+    
+    def get_weight(self):
+        
+        return float(self.weight) if self.weight else 1.0
+
+    def __str__(self):
+        
+        return "Input (weight) "+str(self.id)+": "+str(self.get_target_cell_id())+":"+str(self.get_segment_id())+"("+'PERCENTAGE.6f'PERCENTAGEself.get_fraction_along()+"), weight: "+'PERCENTAGE.6f'PERCENTAGEself.get_weight()
+        
+    ''',
+    class_names=(["InputW"])
+    )
+  
+METHOD_SPECS+=(input_w,)
+
+
 nml_doc_summary = MethodSpec(name='summary',
     source='''\
 
@@ -384,12 +558,17 @@ nml_doc_summary = MethodSpec(name='summary',
             tot_cells = 0 
             pop_info = ""
             for pop in sorted(network.populations, key=lambda x: x.id):
-                pop_info+="*     Population: "+pop.id+" with "+str(pop.size)+" components of type "+pop.component+"\\n"
+                pop_info+="*     "+str(pop)+"\\n"
                 tot_pop+=1
-                tot_cells+=pop.size
+                tot_cells+=pop.get_size()
                 if len(pop.instances)>0:
                     loc = pop.instances[0].location
                     pop_info+="*       Locations: ["+str(loc)+", ...]\\n"
+                if len(pop.properties)>0:
+                    pop_info+="*       Properties: "
+                    for p in pop.properties: 
+                        pop_info+=(str(p.tag)+'='+str(p.value)+'; ')
+                    pop_info+="\\n"
             
             info+="*   "+str(tot_cells)+" cells in "+str(tot_pop)+" populations \\n"+pop_info+"*\\n"
                 
@@ -399,7 +578,7 @@ nml_doc_summary = MethodSpec(name='summary',
             
             proj_info = ""
             for proj in sorted(network.projections, key=lambda x: x.id):
-                proj_info+="*     Projection: "+proj.id+" from "+proj.presynaptic_population+" to "+proj.postsynaptic_population+", synapse: "+proj.synapse+"\\n"
+                proj_info+="*     "+str(proj)+"\\n"
                 tot_proj+=1
                 tot_conns+=len(proj.connections)
                 tot_conns+=len(proj.connection_wds)
@@ -413,10 +592,26 @@ nml_doc_summary = MethodSpec(name='summary',
                 tot_proj+=1
                 tot_conns+=len(proj.electrical_connections)
                 tot_conns+=len(proj.electrical_connection_instances)
+                tot_conns+=len(proj.electrical_connection_instance_ws)
                 if len(proj.electrical_connections)>0:
                     proj_info+="*       "+str(len(proj.electrical_connections))+" connections: [("+str(proj.electrical_connections[0])+"), ...]\\n"
                 if len(proj.electrical_connection_instances)>0:
                     proj_info+="*       "+str(len(proj.electrical_connection_instances))+" connections: [("+str(proj.electrical_connection_instances[0])+"), ...]\\n"
+                if len(proj.electrical_connection_instance_ws)>0:
+                    proj_info+="*       "+str(len(proj.electrical_connection_instance_ws))+" connections: [("+str(proj.electrical_connection_instance_ws[0])+"), ...]\\n"
+                    
+            for proj in sorted(network.continuous_projections, key=lambda x: x.id):
+                proj_info+="*     Continuous projection: "+proj.id+" from "+proj.presynaptic_population+" to "+proj.postsynaptic_population+"\\n"
+                tot_proj+=1
+                tot_conns+=len(proj.continuous_connections)
+                tot_conns+=len(proj.continuous_connection_instances)
+                tot_conns+=len(proj.continuous_connection_instance_ws)
+                if len(proj.continuous_connections)>0:
+                    proj_info+="*       "+str(len(proj.continuous_connections))+" connections: [("+str(proj.continuous_connections[0])+"), ...]\\n"
+                if len(proj.continuous_connection_instances)>0:
+                    proj_info+="*       "+str(len(proj.continuous_connection_instances))+" connections: [("+str(proj.continuous_connection_instances[0])+"), ...]\\n"
+                if len(proj.continuous_connection_instance_ws)>0:
+                    proj_info+="*       "+str(len(proj.continuous_connection_instance_ws))+" connections (w): [("+str(proj.continuous_connection_instance_ws[0])+"), ...]\\n"
                     
             info+="*   "+str(tot_conns)+" connections in "+str(tot_proj)+" projections \\n"+proj_info+"*\\n"
             
@@ -424,11 +619,14 @@ nml_doc_summary = MethodSpec(name='summary',
             tot_inputs = 0
             input_info = ""
             for il in sorted(network.input_lists, key=lambda x: x.id):
-                input_info+="*     Input list: "+il.id+" to "+il.populations+", component "+il.component+"\\n"
+                input_info+="*     "+str(il)+"\\n"
                 tot_input_lists += 1
                 if len(il.input)>0:
                     input_info+="*       "+str(len(il.input))+" inputs: [("+str(il.input[0])+"), ...]\\n"
                     tot_inputs+=len(il.input)
+                if len(il.input_ws)>0:
+                    input_info+="*       "+str(len(il.input_ws))+" inputs: [("+str(il.input_ws[0])+"), ...]\\n"
+                    tot_inputs+=len(il.input_ws)
                     
             info+="*   "+str(tot_inputs)+" inputs in "+str(tot_input_lists)+" input lists \\n"+input_info+"*\\n"
                     
@@ -476,6 +674,12 @@ network_get_by_id = MethodSpec(name='get_by_id',
                         all_ids.append(m.id)
         print("Id "+id+" not found in <network> element. All ids: "+str(all_ids))
         return None
+        
+        
+    def __str__(self):
+        
+        return "Network "+str(self.id)+" with "+str(len(self.populations))+" population(s)"
+        
     ''',
     class_names=("Network")
     )
@@ -497,10 +701,18 @@ cell_methods = MethodSpec(name='cell_methods',
         return segments
         
 
-    def get_ordered_segments_in_groups(self, group_list, check_parentage=False, include_cumulative_lengths=False):
+    def get_ordered_segments_in_groups(self, 
+                                       group_list, 
+                                       check_parentage=False, 
+                                       include_cumulative_lengths=False, 
+                                       include_path_lengths=False, 
+                                       path_length_metric="Path Length from root"): # Only option supported
 
         unord_segs = {}
         other_segs = {}
+        
+        if isinstance(group_list, str):
+            group_list = [group_list]
 
         segments = self.get_segment_ids_vs_segments()
 
@@ -535,7 +747,7 @@ cell_methods = MethodSpec(name='cell_methods',
 
         if check_parentage:
             # check parent ordering
-
+            
             for key in ord_segs.keys():   
                 existing_ids = []
                 for s in ord_segs[key]:
@@ -545,25 +757,79 @@ cell_methods = MethodSpec(name='cell_methods',
                     existing_ids.append(s.id)
 
 
-        if include_cumulative_lengths:
+        if include_cumulative_lengths or include_path_lengths:
             import math
+            
             cumulative_lengths = {}
+            path_lengths_to_proximal = {}
+            path_lengths_to_distal = {}
+            
             for key in ord_segs.keys():   
                 cumulative_lengths[key] = []
+                path_lengths_to_proximal[key] = {}
+                path_lengths_to_distal[key] = {}
+                
                 tot_len = 0
-                for seg in ord_segs[key]:       
+                for seg in ord_segs[key]:     
                     d = seg.distal
                     p = seg.proximal
                     if not p:
                         parent_seg = segments[seg.parent.segments]
                         p = parent_seg.distal
                     length = math.sqrt( (d.x-p.x)**2 + (d.y-p.y)**2 + (d.z-p.z)**2 )
+                    
+                    if not seg.parent or not seg.parent.segments in path_lengths_to_distal[key]:
+                    
+                        path_lengths_to_proximal[key][seg.id] = 0
+                        last_seg = seg
+                        par_seg_element = seg.parent
+                        while par_seg_element!=None:
+                        
+                            par_seg = segments[par_seg_element.segments]
+                            d = par_seg.distal
+                            p = par_seg.proximal
+                            
+                            if not p:
+                                par_seg_parent_seg = segments[par_seg.parent.segments]
+                                p = par_seg_parent_seg.distal
+                                
+                            par_length = math.sqrt( (d.x-p.x)**2 + (d.y-p.y)**2 + (d.z-p.z)**2 )
+                            
+                            fract = float(last_seg.parent.fraction_along)
+                            path_lengths_to_proximal[key][seg.id] += par_length*fract
+                            
+                            last_seg = par_seg
+                            par_seg_element = par_seg.parent
+                            
+                        
+                    else:
+                        pd = path_lengths_to_distal[key][seg.parent.segments]
+                        pp = path_lengths_to_proximal[key][seg.parent.segments]
+                        fract = float(seg.parent.fraction_along)
+                        
+                        path_lengths_to_proximal[key][seg.id] = pp + (pd - pp)*fract
+                        
+                    path_lengths_to_distal[key][seg.id] = path_lengths_to_proximal[key][seg.id] + length
+                    
                     tot_len += length
                     cumulative_lengths[key].append(tot_len)
+                    
+                    
+        if include_path_lengths and not include_cumulative_lengths:
+        
+            return ord_segs, path_lengths_to_proximal, path_lengths_to_distal
 
+        if include_cumulative_lengths and not include_path_lengths:
+        
             return ord_segs, cumulative_lengths
 
+        if include_cumulative_lengths and include_path_lengths:
+        
+            return ord_segs, cumulative_lengths, path_lengths_to_proximal, path_lengths_to_distal
+
         return ord_segs
+        
+        
                 
     def summary(self):
         print("*******************************************************")
@@ -610,7 +876,7 @@ inserts['Network'] = '''
             eproj.exportHdf5(h5file, netGroup)
             
         for cproj in self.continuous_projections:
-            raise Exception("Error: <continuousProjection> not yet supported!")
+            cproj.exportHdf5(h5file, netGroup)
             
         for il in self.input_lists:
             il.exportHdf5(h5file, netGroup)
@@ -624,29 +890,40 @@ inserts['Population'] = '''
         popGroup = h5file.create_group(h5Group, 'population_'+self.id)
         popGroup._f_setattr("id", self.id)
         popGroup._f_setattr("component", self.component)
+        for p in self.properties:
+            popGroup._f_setattr("property:"+p.tag, p.value)
+            
         
         if len(self.instances)>0:
 
-            colCount = 4
-            a = numpy.ones([len(self.instances), colCount], numpy.float32)
+            colCount = 3
+            a = numpy.zeros([len(self.instances), colCount], numpy.float32)
 
             count=0
             for instance in self.instances:
-              a[count,0] = instance.id
-              a[count,1] = instance.location.x
-              a[count,2] = instance.location.y
-              a[count,3] = instance.location.z
+              a[count,0] = instance.location.x
+              a[count,1] = instance.location.y
+              a[count,2] = instance.location.z
 
               count=count+1
-        
+              
             popGroup._f_setattr("size", count)
             popGroup._f_setattr("type", "populationList")
 
-            h5file.create_carray(popGroup, self.id, obj=a, title="Locations of cells in "+ self.id)
+            array = h5file.create_carray(popGroup, self.id, obj=a, title="Locations of cells in "+ self.id)
+            array._f_setattr("column_0", "x")
+            array._f_setattr("column_1", "y")
+            array._f_setattr("column_2", "z")
             
         else:
             popGroup._f_setattr("size", self.size)
         
+    def get_size(self):
+        return len(self.instances) if len(self.instances)>0 else (self.size if self.size else 0)
+    
+    def __str__(self):
+        
+        return "Population: "+str(self.id)+" with "+str( self.get_size() )+" components of type "+(self.component if self.component else "???")
         
 '''
 
@@ -665,7 +942,7 @@ inserts['Projection'] = '''
         
         connection_wds = len(self.connection_wds) > 0
         
-        cols = 3
+        cols = 2
         
         extra_cols = {}
         
@@ -674,11 +951,11 @@ inserts['Projection'] = '''
         include_segment_fraction = has_segment_fraction_info(self.connections) or has_segment_fraction_info(self.connection_wds)
         
         if include_segment_fraction:
+            extra_cols["column_"+str(cols)] = "pre_segment_id"
+            extra_cols["column_"+str(cols+1)] = "post_segment_id"
+            extra_cols["column_"+str(cols+2)] = "pre_fraction_along"
+            extra_cols["column_"+str(cols+3)] = "post_fraction_along"
             cols +=4
-            extra_cols["column_3"] = "pre_segment_id"
-            extra_cols["column_4"] = "post_segment_id"
-            extra_cols["column_5"] = "pre_fraction_along"
-            extra_cols["column_6"] = "post_fraction_along"
             
         
         if connection_wds:
@@ -686,33 +963,31 @@ inserts['Projection'] = '''
             extra_cols["column_"+str(cols+1)] = "delay"
             cols+=2
         
-        
-        a = numpy.ones([len(self.connections)+len(self.connection_wds), cols], numpy.float32)
-        
+        a = numpy.zeros([len(self.connections)+len(self.connection_wds), cols], numpy.float32)
         
         count=0
         
         for connection in self.connections:
-          a[count,0] = connection.id
-          a[count,1] = connection.get_pre_cell_id()
-          a[count,2] = connection.get_post_cell_id()  
+          ####a[count,0] = connection.id
+          a[count,0] = connection.get_pre_cell_id()
+          a[count,1] = connection.get_post_cell_id()  
           if include_segment_fraction:
-            a[count,3] = connection.pre_segment_id  
-            a[count,4] = connection.post_segment_id  
-            a[count,5] = connection.pre_fraction_along 
-            a[count,6] = connection.post_fraction_along          
+            a[count,2] = connection.pre_segment_id  
+            a[count,3] = connection.post_segment_id  
+            a[count,4] = connection.pre_fraction_along 
+            a[count,5] = connection.post_fraction_along          
           count=count+1
           
         for connection in self.connection_wds:
-          a[count,0] = connection.id
-          a[count,1] = connection.get_pre_cell_id()
-          a[count,2] = connection.get_post_cell_id()  
+          ###a[count,0] = connection.id
+          a[count,0] = connection.get_pre_cell_id()
+          a[count,1] = connection.get_post_cell_id()  
           
           if include_segment_fraction:
-            a[count,3] = connection.pre_segment_id  
-            a[count,4] = connection.post_segment_id  
-            a[count,5] = connection.pre_fraction_along 
-            a[count,6] = connection.post_fraction_along  
+            a[count,2] = connection.pre_segment_id  
+            a[count,3] = connection.post_segment_id  
+            a[count,4] = connection.pre_fraction_along 
+            a[count,5] = connection.post_fraction_along  
           
           a[count,cols-2] = connection.weight  
           if 'ms' in connection.delay:
@@ -725,15 +1000,19 @@ inserts['Projection'] = '''
           a[count,cols-1] = delay          
           count=count+1
         
+        if len(a)>0:
+            array = h5file.create_carray(projGroup, self.id, obj=a, title="Connections of cells in "+ self.id)
+
+            ###array._f_setattr("column_0", "id")
+            array._f_setattr("column_0", "pre_cell_id")
+            array._f_setattr("column_1", "post_cell_id")
+
+            for col in extra_cols.keys():
+                array._f_setattr(col,extra_cols[col])
+           
             
-        array = h5file.create_carray(projGroup, self.id, obj=a, title="Connections of cells in "+ self.id)
-        
-        array._f_setattr("column_0", "id")
-        array._f_setattr("column_1", "pre_cell_id")
-        array._f_setattr("column_2", "post_cell_id")
-        
-        for col in extra_cols.keys():
-            array._f_setattr(col,extra_cols[col])
+    def __str__(self):
+        return "Projection: "+self.id+" from "+self.presynaptic_population+" to "+self.postsynaptic_population+", synapse: "+self.synapse
             
         
         
@@ -749,22 +1028,24 @@ inserts['ElectricalProjection'] = '''
         projGroup._f_setattr("presynapticPopulation", self.presynaptic_population)
         projGroup._f_setattr("postsynapticPopulation", self.postsynaptic_population)
         
-        syn = self.electrical_connections[0].synapse if len(self.electrical_connections)>0 else self.electrical_connection_instances[0].synapse
+        syn = self.electrical_connections[0].synapse if len(self.electrical_connections)>0 else \
+                    self.electrical_connection_instances[0].synapse if len(self.electrical_connection_instances)>0 else self.electrical_connection_instance_ws[0].synapse
         projGroup._f_setattr("synapse", syn )
-        
-        
-        
+                
         cols = 7
         extra_cols = {}
         
-        num_tot = len(self.electrical_connections)+len(self.electrical_connection_instances)
+        num_tot = len(self.electrical_connections)+len(self.electrical_connection_instances)+len(self.electrical_connection_instance_ws)
+        if len(self.electrical_connection_instance_ws)>0:
+            extra_cols["column_"+str(cols)] = "weight"
+            cols+=1
         
         #print("Exporting "+str(num_tot)+" electrical connections")
-        a = numpy.ones([num_tot, cols], numpy.float32)
-        
+        a = numpy.zeros([num_tot, cols], numpy.float32)
         
         count=0
         
+        # TODO: optimise for single compartment cells, i.e. where no pre_segment/post_fraction_along etc.
         for connection in self.electrical_connections:
           a[count,0] = connection.id
           a[count,1] = connection.get_pre_cell_id()
@@ -785,6 +1066,17 @@ inserts['ElectricalProjection'] = '''
           a[count,6] = connection.post_fraction_along          
           count=count+1
           
+        for connection in self.electrical_connection_instance_ws:
+          a[count,0] = connection.id
+          a[count,1] = connection.get_pre_cell_id()
+          a[count,2] = connection.get_post_cell_id()  
+          a[count,3] = connection.pre_segment  
+          a[count,4] = connection.post_segment  
+          a[count,5] = connection.pre_fraction_along 
+          a[count,6] = connection.post_fraction_along    
+          a[count,7] = connection.get_weight()          
+          count=count+1
+          
         array = h5file.create_carray(projGroup, self.id, obj=a, title="Connections of cells in "+ self.id)
         
         array._f_setattr("column_0", "id")
@@ -794,7 +1086,90 @@ inserts['ElectricalProjection'] = '''
         array._f_setattr("column_4", "post_segment_id")
         array._f_setattr("column_5", "pre_fraction_along")
         array._f_setattr("column_6", "post_fraction_along")
+
+        for col in extra_cols.keys():
+            array._f_setattr(col,extra_cols[col])
         
+'''
+
+
+
+inserts['ContinuousProjection'] = '''
+         
+        import numpy
+        
+        projGroup = h5file.create_group(h5Group, 'projection_'+self.id)
+        projGroup._f_setattr("id", self.id)
+        projGroup._f_setattr("type", "continuousProjection")
+        projGroup._f_setattr("presynapticPopulation", self.presynaptic_population)
+        projGroup._f_setattr("postsynapticPopulation", self.postsynaptic_population)
+        
+        pre_comp = self.continuous_connections[0].pre_component if len(self.continuous_connections)>0 else \
+                            self.continuous_connection_instances[0].pre_component if len(self.continuous_connection_instances)>0 else self.continuous_connection_instance_ws[0].pre_component
+        projGroup._f_setattr("preComponent", pre_comp )
+        post_comp = self.continuous_connections[0].post_component if len(self.continuous_connections)>0 else \
+                            self.continuous_connection_instances[0].post_component if len(self.continuous_connection_instances)>0 else self.continuous_connection_instance_ws[0].post_component
+        projGroup._f_setattr("postComponent", post_comp )
+                
+        cols = 7
+        extra_cols = {}
+        
+        num_tot = len(self.continuous_connections)+len(self.continuous_connection_instances)+len(self.continuous_connection_instance_ws)
+        
+        if len(self.continuous_connection_instance_ws)>0:
+            extra_cols["column_"+str(cols)] = 'weight'
+            cols+=1
+        
+        #print("Exporting "+str(num_tot)+" continuous connections")
+        a = numpy.zeros([num_tot, cols], numpy.float32)
+        
+        count=0
+        
+        # TODO: optimise for single compartment cells, i.e. where no pre_segment/post_fraction_along etc.
+        for connection in self.continuous_connections:
+          a[count,0] = connection.id
+          a[count,1] = connection.get_pre_cell_id()
+          a[count,2] = connection.get_post_cell_id()  
+          a[count,3] = connection.pre_segment  
+          a[count,4] = connection.post_segment  
+          a[count,5] = connection.pre_fraction_along 
+          a[count,6] = connection.post_fraction_along          
+          count=count+1
+          
+        for connection in self.continuous_connection_instances:
+          a[count,0] = connection.id
+          a[count,1] = connection.get_pre_cell_id()
+          a[count,2] = connection.get_post_cell_id()  
+          a[count,3] = connection.pre_segment  
+          a[count,4] = connection.post_segment  
+          a[count,5] = connection.pre_fraction_along 
+          a[count,6] = connection.post_fraction_along          
+          count=count+1
+          
+          
+        for connection in self.continuous_connection_instance_ws:
+          a[count,0] = connection.id
+          a[count,1] = connection.get_pre_cell_id()
+          a[count,2] = connection.get_post_cell_id()  
+          a[count,3] = connection.pre_segment  
+          a[count,4] = connection.post_segment  
+          a[count,5] = connection.pre_fraction_along 
+          a[count,6] = connection.post_fraction_along  
+          a[count,7] = connection.weight          
+          count=count+1
+          
+          
+        array = h5file.create_carray(projGroup, self.id, obj=a, title="Connections of cells in "+ self.id)
+        
+        array._f_setattr("column_0", "id")
+        array._f_setattr("column_1", "pre_cell_id")
+        array._f_setattr("column_2", "post_cell_id")
+        array._f_setattr("column_3", "pre_segment_id")
+        array._f_setattr("column_4", "post_segment_id")
+        array._f_setattr("column_5", "pre_fraction_along")
+        array._f_setattr("column_6", "post_fraction_along")
+        for k in extra_cols:
+            array._f_setattr(k, extra_cols[k])
             
         
 '''
@@ -809,16 +1184,35 @@ inserts['InputList'] = '''
         ilGroup._f_setattr("component", self.component)
         ilGroup._f_setattr("population", self.populations)
         
-        colCount = 4
-        a = numpy.ones([len(self.input), colCount], numpy.float32)
+        cols = 4
+        
+        extra_cols = {}
+        
+        num_tot = len(self.input)+len(self.input_ws)
+        
+        if len(self.input_ws)>0:
+            extra_cols["column_"+str(cols)] = 'weight'
+            cols+=1
+        
+        #print("Exporting "+str(num_tot)+" inputs")
+        a = numpy.zeros([num_tot, cols], numpy.float32)
         
         count=0
+        
         for input in self.input:
             a[count,0] = input.id
             a[count,1] = input.get_target_cell_id()
             a[count,2] = input.get_segment_id()
             a[count,3] = input.get_fraction_along()
             count+=1
+        
+        for input in self.input_ws:
+            a[count,0] = input.id
+            a[count,1] = input.get_target_cell_id()
+            a[count,2] = input.get_segment_id()
+            a[count,3] = input.get_fraction_along()
+            a[count,4] = input.get_weight()
+            count+=1
             
         array = h5file.create_carray(ilGroup, self.id, obj=a, title="Locations of inputs in "+ self.id)
             
@@ -826,6 +1220,12 @@ inserts['InputList'] = '''
         array._f_setattr("column_1", "target_cell_id")
         array._f_setattr("column_2", "segment_id")
         array._f_setattr("column_3", "fraction_along")
+        for k in extra_cols:
+            array._f_setattr(k, extra_cols[k])
+        
+    def __str__(self):
+        
+        return "Input list: "+self.id+" to "+self.populations+", component "+self.component
         
 '''
 
